"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var Lint = require("tslint");
var RuleId_1 = require("./RuleId");
var Rule = /** @class */ (function (_super) {
    __extends(Rule, _super);
    function Rule() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Rule.prototype.apply = function (sourceFile) {
        var walker = new ImportsWalker(sourceFile, this.getOptions());
        this.applyWithWalker(walker);
        return walker.getFailures();
    };
    // include the rule ID, to make it easier to disable
    Rule.FAILURE_STRING = "do not import from invalid folder like 'DISALLOWED_TEXT' (" + RuleId_1.RuleId.TsfFoldersImportFromDisallowedFolders + ")";
    return Rule;
}(Lint.Rules.AbstractRule));
exports.Rule = Rule;
var ImportsWalker = /** @class */ (function (_super) {
    __extends(ImportsWalker, _super);
    function ImportsWalker() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    ImportsWalker.prototype.visitImportDeclaration = function (node) {
        this.validate(node, node.moduleSpecifier.getText());
    };
    ImportsWalker.prototype.visitImportEqualsDeclaration = function (node) {
        this.validate(node, node.moduleReference.getText());
        _super.prototype.visitImportEqualsDeclaration.call(this, node);
    };
    ImportsWalker.prototype.validate = function (node, text) {
        var options = this.getOptions();
        if (options.length !== 1) {
            throw new Error("tslint rule is misconfigured (" + RuleId_1.RuleId.TsfFoldersImportFromDisallowedFolders + ")");
        }
        var disallowedTexts = this.getOptions()[0].disallowed;
        for (var _i = 0, disallowedTexts_1 = disallowedTexts; _i < disallowedTexts_1.length; _i++) {
            var disallowed = disallowedTexts_1[_i];
            if (text.indexOf(disallowed) >= 0) {
                this.addFailureAtNode(node, Rule.FAILURE_STRING.replace("DISALLOWED_TEXT", disallowed));
                // one error per line is enough!
                return;
            }
        }
    };
    return ImportsWalker;
}(Lint.RuleWalker));
//# sourceMappingURL=tsfFoldersImportFromDisallowedFoldersRule.js.map