"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var Lint = require("tslint");
var ConfigFactory_1 = require("./config/ConfigFactory");
var RuleId_1 = require("./RuleId");
var GeneralRuleUtils_1 = require("./utils/GeneralRuleUtils");
var ImportRuleUtils_1 = require("./utils/ImportRuleUtils");
var DISALLOW_IMPORT_FROM_SELF_MESSAGE = "do not import a package from itself - use a relative path";
var DISALLOW_IMPORT_FROM_BANNED_MESSAGE = "do not use a banned import path from package";
var Rule = /** @class */ (function (_super) {
    __extends(Rule, _super);
    function Rule() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Rule.prototype.apply = function (sourceFile) {
        var walker = new ImportsWalker(sourceFile, this.getOptions());
        this.applyWithWalker(walker);
        return walker.getFailures();
    };
    return Rule;
}(Lint.Rules.AbstractRule));
exports.Rule = Rule;
var ImportsWalker = /** @class */ (function (_super) {
    __extends(ImportsWalker, _super);
    function ImportsWalker() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    ImportsWalker.prototype.visitImportDeclaration = function (node) {
        this.validate(node, node.moduleSpecifier.getText());
    };
    ImportsWalker.prototype.visitImportEqualsDeclaration = function (node) {
        this.validate(node, node.moduleReference.getText());
        _super.prototype.visitImportEqualsDeclaration.call(this, node);
    };
    ImportsWalker.prototype.validate = function (node, text) {
        // algorithm:
        /*
        - determine this files PackageFolder, PackageSubFolder
        - if ThirdParty then skip
        - determine the PackageFolder, PackageSubFolder of the import
        - if ThirdParty then skip
        - disallowImportFromSelf -> disallow if same
        - check if the import is allowed
        */
        var config = ConfigFactory_1.ConfigFactory.createForBetweenPackages(this.getOptions());
        var thisPackageLocation = ImportRuleUtils_1.ImportRuleUtils.determinePackageLocationFromPath(node.getSourceFile().fileName, RuleId_1.RuleId.TsfFoldersImportsBetweenPackages, config, ImportRuleUtils_1.PathSource.SourceFilePath);
        if (ImportRuleUtils_1.ImportRuleUtils.isThisPackageThirdParty(thisPackageLocation, node)) {
            return;
        }
        if (!thisPackageLocation.packageFolder) {
            throw new Error("unexpected: this package is not ThirdParty, but has no packageFolder in its location");
        }
        var importPackageLocation = ImportRuleUtils_1.ImportRuleUtils.determinePackageLocationFromPath(text, RuleId_1.RuleId.TsfFoldersImportsBetweenPackages, config, ImportRuleUtils_1.PathSource.ImportText, thisPackageLocation);
        ImportRuleUtils_1.ImportRuleUtils.logPackageAndImport(node, thisPackageLocation, importPackageLocation);
        var isImportRecognised = !ImportRuleUtils_1.ImportRuleUtils.isPackageThirdParty(importPackageLocation);
        if (isImportRecognised &&
            importPackageLocation.packageName === thisPackageLocation.packageName &&
            config.disallowImportFromSelf.enabled &&
            !ImportRuleUtils_1.ImportRuleUtils.shouldIgnoreFile(node, config.disallowImportFromSelf.ignorePaths)) {
            this.addFailureAtNode(node, GeneralRuleUtils_1.GeneralRuleUtils.buildFailureString(DISALLOW_IMPORT_FROM_SELF_MESSAGE, RuleId_1.RuleId.TsfFoldersImportsBetweenPackages));
            return;
        }
        if (isImportRecognised &&
            config.checkImportsBetweenPackages.enabled &&
            !ImportRuleUtils_1.ImportRuleUtils.shouldIgnoreFile(node, config.checkImportsBetweenPackages.ignorePaths)) {
            if (!thisPackageLocation.packageFolder || !importPackageLocation.packageFolder) {
                return;
            }
            if (this.hasBannedImportPattern(config.checkImportsBetweenPackages, text, node)) {
                return;
            }
            if (importPackageLocation.packageFolder === thisPackageLocation.packageFolder) {
                if (!config.checkImportsBetweenPackages.checkSubFoldersEnabled) {
                    return;
                }
                // TODO xxx extract fun?
                // check sub-folders
                if (thisPackageLocation.packageSubFolder &&
                    importPackageLocation.packageSubFolder &&
                    thisPackageLocation.packageSubFolder.importPath !==
                        importPackageLocation.packageSubFolder.importPath) {
                    if (thisPackageLocation.packageSubFolder.allowedToImport.some(function (allowed) { return allowed === "*"; })) {
                        return;
                    }
                    if (!thisPackageLocation.packageSubFolder.allowedToImport.some(function (allowed) {
                        return importPackageLocation.packageSubFolder.importPath === allowed;
                    })) {
                        var failureMessage = "'" + thisPackageLocation.packageName + "' sub folder '" + thisPackageLocation.packageSubFolder.importPath + "' is not allowed to import from '" + importPackageLocation.packageSubFolder.importPath + "'";
                        this.addFailureAtNodeWithMessage(node, failureMessage);
                    }
                }
            }
            else {
                var thisPackageFolder = thisPackageLocation.packageFolder;
                if (thisPackageFolder.allowedToImport.find(function (allowed) { return allowed === "*"; })) {
                    return;
                }
                if (!thisPackageFolder.allowedToImport.find(function (allowed) { return allowed === importPackageLocation.packageName; })) {
                    var failureMessage = "'" + thisPackageLocation.packageName + "' is not allowed to import from '" + importPackageLocation.packageName + "'";
                    this.addFailureAtNodeWithMessage(node, failureMessage);
                }
            }
        }
    };
    ImportsWalker.prototype.hasBannedImportPattern = function (checkImportsBetweenPackages, text, node) {
        var bannedImports = this.buildListOfBannedImports(checkImportsBetweenPackages);
        if (bannedImports && bannedImports.some(function (ban) { return text.indexOf(ban) >= 0; })) {
            this.addFailureAtNode(node, GeneralRuleUtils_1.GeneralRuleUtils.buildFailureString(DISALLOW_IMPORT_FROM_BANNED_MESSAGE, RuleId_1.RuleId.TsfFoldersImportsBetweenPackages));
            return true;
        }
        return false;
    };
    ImportsWalker.prototype.buildListOfBannedImports = function (checkImportsBetweenPackages) {
        var ban = checkImportsBetweenPackages.ban, banBlacklist = checkImportsBetweenPackages.banBlacklist, packages = checkImportsBetweenPackages.packages;
        if (!ban) {
            return [];
        }
        var bannedImports = [];
        packages
            .filter(function (pkg) { return !banBlacklist || !banBlacklist.some(function (b) { return pkg.importPath === b; }); })
            .forEach(function (pkg) {
            ban.forEach(function (b) {
                bannedImports.push(b.replace("{PACKAGE}", pkg.importPath));
            });
        });
        return bannedImports;
    };
    ImportsWalker.prototype.addFailureAtNodeWithMessage = function (node, failureMessage) {
        this.addFailureAtNode(node, GeneralRuleUtils_1.GeneralRuleUtils.buildFailureString(failureMessage, RuleId_1.RuleId.TsfFoldersImportsBetweenPackages));
    };
    return ImportsWalker;
}(Lint.RuleWalker));
//# sourceMappingURL=tsfFoldersImportsBetweenPackagesRule.js.map