(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('path'), require('tslint')) :
  typeof define === 'function' && define.amd ? define(['exports', 'path', 'tslint'], factory) :
  (factory((global.tslintFolders = {}),global.path,global.Lint));
}(this, (function (exports,path,Lint) { 'use strict';

  function getDefaultDisabledTestRuleConfig() {
      return {
          ban: ["describe.only", "it.only", "describe.skip", "it.skip", "fdescribe", "fit", "xit", "xdescribe"],
          includePaths: [".spec.ts", ".spec.tsx", ".ispec.ts", ".ispec.tsx", ".spec.karma.tsx"]
      };
  }

  function getDefaultBreakpointRuleConfig() {
      return {
          debugTokens: ["browser.debug"],
          includePaths: [".spec.ts", ".spec.tsx", ".ispec.ts", ".ispec.tsx"]
      };
  }

  var RuleId;
  (function (RuleId) {
      RuleId["TsfFoldersDisabledTest"] = "tsf-folders-disabled-test";
      RuleId["TsfFoldersFileNames"] = "tsf-folders-file-names";
      RuleId["TsfFoldersImportFromDisallowedFolders"] = "tsf-folders-import-from-disallowed-folders";
      RuleId["TsfFoldersImportsBetweenPackages"] = "tsf-folders-imports-between-packages";
      RuleId["TsfFoldersTestWithBreakpoint"] = "tsf-folders-test-with-breakpoint";
  })(RuleId || (RuleId = {}));

  (function (ConfigFactory) {
      function createForBetweenPackages(options) {
          var config = create(options, RuleId.TsfFoldersImportsBetweenPackages);
          // TODO xxx validate a bit (at least importPath should be set, allowedToImport should refer to recognised importPath)
          return config;
      }
      ConfigFactory.createForBetweenPackages = createForBetweenPackages;
      function createForFilenames(options) {
          var config = createFromArguments(options, RuleId.TsfFoldersFileNames);
          config.casings = config["file-name-casing"];
          validate(config, "casings", RuleId.TsfFoldersFileNames);
          validate(config, "ignorePaths", RuleId.TsfFoldersFileNames);
          return config;
      }
      ConfigFactory.createForFilenames = createForFilenames;
      function createForDisabledTestRule(options) {
          console.log("options.ruleArguments", options.ruleArguments);
          // older config had just 'true':
          if (options.ruleArguments.length === 0) {
              return getDefaultDisabledTestRuleConfig();
          }
          var config = createFromArguments(options, RuleId.TsfFoldersDisabledTest);
          validate(config, "ban", RuleId.TsfFoldersDisabledTest);
          validate(config, "includePaths", RuleId.TsfFoldersDisabledTest);
          return config;
      }
      ConfigFactory.createForDisabledTestRule = createForDisabledTestRule;
      function createForTestBreakpointRule(options) {
          // older config had just 'true':
          if (options.ruleArguments.length === 0) {
              return getDefaultBreakpointRuleConfig();
          }
          var config = createFromArguments(options, RuleId.TsfFoldersTestWithBreakpoint);
          validate(config, "debugTokens", RuleId.TsfFoldersTestWithBreakpoint);
          validate(config, "includePaths", RuleId.TsfFoldersTestWithBreakpoint);
          return config;
      }
      ConfigFactory.createForTestBreakpointRule = createForTestBreakpointRule;
      function validate(config, prop, ruleId) {
          if (config[prop] === undefined) {
              throw new Error("invalid config for rule " + ruleId + " - " + prop + " is missing");
          }
      }
      function create(options, ruleId) {
          if (options.length !== 1) {
              throw new Error("tslint rule is misconfigured (" + ruleId + ") - options length is " + options.length);
          }
          var config = options[0].config;
          return config;
      }
      function createFromArguments(options, ruleId) {
          var args = options.ruleArguments;
          if (args.length !== 1) {
              throw new Error("tslint rule is misconfigured (" + ruleId + ") - options.ruleArguments length is " + args.length);
          }
          var config = args[0];
          return config;
      }
  })(exports.ConfigFactory || (exports.ConfigFactory = {}));

  /*! *****************************************************************************
  Copyright (c) Microsoft Corporation. All rights reserved.
  Licensed under the Apache License, Version 2.0 (the "License"); you may not use
  this file except in compliance with the License. You may obtain a copy of the
  License at http://www.apache.org/licenses/LICENSE-2.0

  THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
  KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
  WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
  MERCHANTABLITY OR NON-INFRINGEMENT.

  See the Apache Version 2.0 License for specific language governing permissions
  and limitations under the License.
  ***************************************************************************** */
  /* global Reflect, Promise */

  var extendStatics = function(d, b) {
      extendStatics = Object.setPrototypeOf ||
          ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
          function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
      return extendStatics(d, b);
  };

  function __extends(d, b) {
      extendStatics(d, b);
      function __() { this.constructor = d; }
      d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
  }

  var GeneralRuleUtils;
  (function (GeneralRuleUtils) {
      function buildFailureString(message, ruleId) {
          // include the rule ID, to make it easier to disable
          return message + " (" + ruleId + ")";
      }
      GeneralRuleUtils.buildFailureString = buildFailureString;
      function isFileInPaths(filePath, paths) {
          return paths.some(function (path$$1) { return filePath.indexOf(path$$1) >= 0; });
      }
      GeneralRuleUtils.isFileInPaths = isFileInPaths;
  })(GeneralRuleUtils || (GeneralRuleUtils = {}));

  // note: MUST be named exactly 'Rule' to be picked up by tslint
  var Rule = /** @class */ (function (_super) {
      __extends(Rule, _super);
      function Rule() {
          return _super !== null && _super.apply(this, arguments) || this;
      }
      Rule.prototype.apply = function (sourceFile) {
          var config = exports.ConfigFactory.createForDisabledTestRule(this.getOptions());
          if (!GeneralRuleUtils.isFileInPaths(sourceFile.fileName, config.includePaths)) {
              return [];
          }
          var walker = new StatementsWalker(sourceFile, this.getOptions(), config);
          this.applyWithWalker(walker);
          return walker.getFailures();
      };
      return Rule;
  }(Lint.Rules.AbstractRule));
  var StatementsWalker = /** @class */ (function (_super) {
      __extends(StatementsWalker, _super);
      function StatementsWalker(sourceFile, options, config) {
          var _this = _super.call(this, sourceFile, options) || this;
          _this.config = config;
          return _this;
      }
      StatementsWalker.prototype.visitCallExpression = function (node) {
          var text = node.getText();
          if (this.config.ban.some(function (token) { return text.startsWith(token); })) {
              this.addFailureAtNode(node.getFirstToken(), GeneralRuleUtils.buildFailureString("do not disable or enable only some tests", RuleId.TsfFoldersDisabledTest));
          }
          _super.prototype.visitCallExpression.call(this, node);
      };
      return StatementsWalker;
  }(Lint.RuleWalker));

  var tsfFoldersDisabledTestRule = /*#__PURE__*/Object.freeze({
    Rule: Rule
  });

  var Casing;
  (function (Casing) {
      Casing["CamelCase"] = "camel-case";
      Casing["PascalCase"] = "pascal-case";
      Casing["KebabCase"] = "kebab-case";
      Casing["SnakeCase"] = "snake-case";
  })(Casing || (Casing = {}));

  var EnumUtils;
  (function (EnumUtils) {
      function parseCasing(casingString) {
          return casingString;
      }
      EnumUtils.parseCasing = parseCasing;
  })(EnumUtils || (EnumUtils = {}));

  var FilenameCasingUtils;
  (function (FilenameCasingUtils) {
      function isCased(filename, casing) {
          if (hasInvalidChars(filename)) {
              return false;
          }
          switch (casing) {
              case Casing.CamelCase:
                  return isCamelCased(filename);
              case Casing.KebabCase:
                  return isKebabCased(filename);
              case Casing.PascalCase:
                  return isPascalCased(filename);
              case Casing.SnakeCase:
                  return isSnakeCased(filename);
              default:
                  throw new Error("unhandled casing " + casing);
          }
      }
      FilenameCasingUtils.isCased = isCased;
      function hasInvalidChars(filename) {
          return /[^a-z^A-Z^0-9._/.-]+/.test(filename);
      }
      function isPascalCased(name) {
          return isUpperCase(name[0]) && !name.includes("_") && !name.includes("-");
      }
      function isCamelCased(name) {
          return isLowerCase(name[0]) && !name.includes("_") && !name.includes("-");
      }
      function isNotSeparatorCased(name, disallowedSeparator) {
          for (var i = 0; i < name.length; i++) {
              var c = name.charAt(i);
              if (c === disallowedSeparator || !isLowerCase(c)) {
                  return false;
              }
          }
          return true;
      }
      function isKebabCased(name) {
          return isNotSeparatorCased(name, "_") && name.includes("-");
      }
      function isSnakeCased(name) {
          return isNotSeparatorCased(name, "-") && name.includes("_");
      }
      function isUpperCase(str) {
          return str === str.toUpperCase();
      }
      function isLowerCase(str) {
          return str === str.toLowerCase();
      }
  })(FilenameCasingUtils || (FilenameCasingUtils = {}));

  var PackageConfigHelper;
  (function (PackageConfigHelper) {
      function getPackage(config, importPath) {
          if (!hasPackage(config, importPath)) {
              throw new Error("config does not have a PackageFolder for importPath '" + importPath + "'");
          }
          var packages = config.checkImportsBetweenPackages.packages.filter(function (pkg) { return pkg.importPath === importPath; });
          if (packages.length === 1) {
              return packages[0];
          }
          if (packages.length > 1) {
              throw new Error("config has multiple PackageFolder's for importPath '" + importPath + "'");
          }
          throw new Error("config does not have a PackageFolder for importPath '" + importPath + "'");
      }
      PackageConfigHelper.getPackage = getPackage;
      function hasPackage(config, importPath) {
          return config.checkImportsBetweenPackages.packages.some(function (pkg) { return pkg.importPath === importPath; });
      }
      PackageConfigHelper.hasPackage = hasPackage;
  })(PackageConfigHelper || (PackageConfigHelper = {}));

  var PathSource;
  (function (PathSource) {
      PathSource[PathSource["SourceFilePath"] = 0] = "SourceFilePath";
      PathSource[PathSource["ImportText"] = 1] = "ImportText";
  })(PathSource || (PathSource = {}));
  var ImportRuleUtils;
  (function (ImportRuleUtils) {
      ImportRuleUtils.IS_DEBUG_ENABLED = false;
      function determinePackageLocationFromPath(filePath, ruleId, config, pathSource, thisPackageLocation) {
          var dirs = cleanPath(filePath).split("/");
          if (ImportRuleUtils.IS_DEBUG_ENABLED) {
              console.log("\nchecking " + PathSource[pathSource] + " against path:", dirs.join(","));
          }
          var packageName = determinePackageName(config, dirs, pathSource);
          if (packageName === null ||
              (pathSource === PathSource.SourceFilePath &&
                  !PackageConfigHelper.hasPackage(config, packageName))) {
              return {
                  packageName: filePath
              };
          }
          var packageFolderAndName = determinePackageFolderAndName(config, dirs, filePath, thisPackageLocation, packageName, pathSource, ruleId);
          packageName = packageFolderAndName.packageName;
          if (!packageFolderAndName.packageFolder) {
              return {
                  packageName: packageName
              };
          }
          if (packageFolderAndName.packageFolder.subFolders.length === 0) {
              return {
                  packageName: packageName,
                  packageFolder: packageFolderAndName.packageFolder
              };
          }
          return determinePackageLocationWithSubFolder(dirs, packageName, packageFolderAndName.packageFolder, pathSource);
      }
      ImportRuleUtils.determinePackageLocationFromPath = determinePackageLocationFromPath;
      // TODO xxx review - too many params!
      function determinePackageFolderAndName(config, dirs, filePath, thisPackageLocation, packageName, pathSource, ruleId) {
          var activePackageName = packageName;
          var packageFolder;
          if (pathSource === PathSource.ImportText && isRelativeImport(dirs[0])) {
              if (!thisPackageLocation) {
                  throw new Error("for path source = ImportText - expect thisPackageLocation to be set");
              }
              // assumption: we are importing from 'this' package
              // (assuming is not ..'ing back into other package! - that would require 'virtual directories')
              packageFolder = thisPackageLocation.packageFolder;
          }
          else {
              try {
                  packageFolder = PackageConfigHelper.getPackage(config, packageName);
              }
              catch (error) {
                  GeneralRuleUtils.buildFailureString("unexpected: import had a recognised package: [" + packageName + "] but could not find the PackageFolder in the config", ruleId);
                  packageFolder = undefined;
                  activePackageName = filePath;
              }
          }
          return {
              packageFolder: packageFolder,
              packageName: activePackageName
          };
      }
      function determinePackageName(config, dirs, pathSource) {
          var packageName = null;
          switch (pathSource) {
              case PathSource.ImportText:
                  {
                      // take the 1st part of the path:
                      // (ignore local directories that happen to have same name as a package)
                      packageName = dirs[0];
                  }
                  break;
              case PathSource.SourceFilePath: {
                  dirs.forEach(function (dir) {
                      if (PackageConfigHelper.hasPackage(config, dir)) {
                          if (packageName === null) {
                              // take the 1st recognised folder:
                              packageName = dir;
                          }
                          else if (ImportRuleUtils.IS_DEBUG_ENABLED) {
                              // this can occur with package names like 'utils'
                              console.warn("import has more than one recognised package: [" + packageName + "," + dir + "]");
                          }
                      }
                  });
                  break;
              }
              default: {
                  throw new Error("unhandled PathSource " + pathSource);
              }
          }
          return packageName;
      }
      function determinePackageLocationWithSubFolder(dirs, packageName, packageFolder, pathSource) {
          var packageSubFolder;
          var activePackageName = packageName;
          switch (pathSource) {
              case PathSource.ImportText:
                  {
                      // take the 1st part of the path:
                      // (ignore local directories that happen to have same name as a package)
                      activePackageName = dirs[0];
                      var isImportingFromSubFolder = isRelativeImport(dirs[0]);
                      if (isImportingFromSubFolder) {
                          var _loop_1 = function (i) {
                              var subFolder = packageFolder.subFolders.find(function (folder) { return folder.importPath === dirs[i]; });
                              if (subFolder) {
                                  packageSubFolder = subFolder;
                                  activePackageName = subFolder.importPath;
                                  return "break";
                              }
                          };
                          for (var i = 1; i < dirs.length; i++) {
                              var state_1 = _loop_1(i);
                              if (state_1 === "break")
                                  break;
                          }
                      }
                  }
                  break;
              case PathSource.SourceFilePath: {
                  var hasPackageDir_1 = false;
                  dirs.forEach(function (dir) {
                      if (packageSubFolder) {
                          return;
                      }
                      if (packageFolder.importPath === dir) {
                          hasPackageDir_1 = true;
                          return;
                      }
                      {
                          if (hasPackageDir_1) {
                              var subFolder = packageFolder.subFolders.find(function (folder) { return folder.importPath === dir; });
                              if (subFolder) {
                                  packageSubFolder = subFolder;
                                  return;
                              }
                          }
                      }
                  });
                  break;
              }
              default: {
                  throw new Error("unhandled PathSource " + pathSource);
              }
          }
          return {
              packageName: activePackageName,
              packageFolder: packageFolder,
              packageSubFolder: packageSubFolder
          };
      }
      function isRelativeImport(pathDir) {
          return pathDir === "." || pathDir === "..";
      }
      ImportRuleUtils.isRelativeImport = isRelativeImport;
      function cleanPath(filePath) {
          var cleaned = filePath.trim();
          cleaned = cleaned.replace(/['"]+/g, "");
          cleaned = cleaned.replace(/[\\]+/g, "\\");
          return cleaned;
      }
      function isThisPackageThirdParty(thisFolderLocation, node) {
          if (isPackageThirdParty(thisFolderLocation)) {
              if (ImportRuleUtils.IS_DEBUG_ENABLED) {
                  console.log(node.getSourceFile().fileName, "- this package ThirdParty!", thisFolderLocation.packageName);
              }
              return true;
          }
          return false;
      }
      ImportRuleUtils.isThisPackageThirdParty = isThisPackageThirdParty;
      function isPackageThirdParty(folderLocation) {
          return !folderLocation.packageFolder && !folderLocation.packageSubFolder;
      }
      ImportRuleUtils.isPackageThirdParty = isPackageThirdParty;
      function logPackageAndImport(node, thisPackageLocation, importPackageLocation) {
          if (ImportRuleUtils.IS_DEBUG_ENABLED) {
              console.log(node.getSourceFile().fileName, "- this package:", thisPackageLocation.packageName, getPackageName(thisPackageLocation.packageFolder), "- import:", importPackageLocation.packageName, getPackageName(importPackageLocation.packageFolder));
          }
      }
      ImportRuleUtils.logPackageAndImport = logPackageAndImport;
      function getPackageName(packageFolder) {
          return packageFolder ? packageFolder.importPath : "(unknown)";
      }
      function shouldIgnoreFile(node, ignorePaths) {
          var filePath = node.getSourceFile().fileName;
          return shouldIgnorePath(filePath, ignorePaths);
      }
      ImportRuleUtils.shouldIgnoreFile = shouldIgnoreFile;
      function shouldIgnorePath(path$$1, ignorePaths) {
          return ignorePaths.some(function (ignore) { return path$$1.indexOf(ignore) >= 0; });
      }
      ImportRuleUtils.shouldIgnorePath = shouldIgnorePath;
  })(ImportRuleUtils || (ImportRuleUtils = {}));

  /** Custom version of the standard file-name-casing rule, that allows for *multiple* casings.
   * ref: https://github.com/palantir/tslint/blob/master/src/rules/fileNameCasingRule.ts
   */
  var Rule$1 = /** @class */ (function (_super) {
      __extends(Rule, _super);
      function Rule() {
          return _super !== null && _super.apply(this, arguments) || this;
      }
      Rule.FAILURE_STRING = function (expectedCasings) {
          var _this = this;
          var stylizedCasings = expectedCasings
              .map(function (casing) { return _this.stylizedNameForCasing(casing); })
              .join(" or ");
          // include the rule ID, to make it easier to disable
          return "File name must be " + stylizedCasings + " (" + RuleId.TsfFoldersFileNames + ")";
      };
      Rule.stylizedNameForCasing = function (casing) {
          switch (casing) {
              case Casing.CamelCase:
                  return "camelCase";
              case Casing.PascalCase:
                  return "PascalCase";
              case Casing.KebabCase:
                  return "kebab-case";
              case Casing.SnakeCase:
                  return "snake_case";
              default:
                  throw new Error("unhandled casing " + casing);
          }
      };
      Rule.prototype.apply = function (sourceFile) {
          var config = exports.ConfigFactory.createForFilenames(this.getOptions());
          if (ImportRuleUtils.shouldIgnorePath(sourceFile.fileName, config.ignorePaths)) {
              return [];
          }
          var parsedPath = path.parse(sourceFile.fileName);
          var filename = parsedPath.name;
          var isAnAllowedCasing = config.casings.some(function (casingName) {
              var casing = EnumUtils.parseCasing(casingName);
              var isAllowed = FilenameCasingUtils.isCased(filename, casing);
              return isAllowed;
          });
          if (!isAnAllowedCasing) {
              return [
                  new Lint.RuleFailure(sourceFile, 0, 0, Rule.FAILURE_STRING(config.casings), RuleId.TsfFoldersFileNames)
              ];
          }
          return [];
      };
      return Rule;
  }(Lint.Rules.AbstractRule));

  var tsfFoldersFileNamesRule = /*#__PURE__*/Object.freeze({
    Rule: Rule$1
  });

  var Rule$2 = /** @class */ (function (_super) {
      __extends(Rule, _super);
      function Rule() {
          return _super !== null && _super.apply(this, arguments) || this;
      }
      Rule.prototype.apply = function (sourceFile) {
          var walker = new ImportsWalker(sourceFile, this.getOptions());
          this.applyWithWalker(walker);
          return walker.getFailures();
      };
      // include the rule ID, to make it easier to disable
      Rule.FAILURE_STRING = "do not import from invalid folder like 'DISALLOWED_TEXT' (" + RuleId.TsfFoldersImportFromDisallowedFolders + ")";
      return Rule;
  }(Lint.Rules.AbstractRule));
  var ImportsWalker = /** @class */ (function (_super) {
      __extends(ImportsWalker, _super);
      function ImportsWalker() {
          return _super !== null && _super.apply(this, arguments) || this;
      }
      ImportsWalker.prototype.visitImportDeclaration = function (node) {
          this.validate(node, node.moduleSpecifier.getText());
      };
      ImportsWalker.prototype.visitImportEqualsDeclaration = function (node) {
          this.validate(node, node.moduleReference.getText());
          _super.prototype.visitImportEqualsDeclaration.call(this, node);
      };
      ImportsWalker.prototype.validate = function (node, text) {
          var options = this.getOptions();
          if (options.length !== 1) {
              throw new Error("tslint rule is misconfigured (" + RuleId.TsfFoldersImportFromDisallowedFolders + ")");
          }
          var disallowedTexts = this.getOptions()[0].disallowed;
          for (var _i = 0, disallowedTexts_1 = disallowedTexts; _i < disallowedTexts_1.length; _i++) {
              var disallowed = disallowedTexts_1[_i];
              if (text.indexOf(disallowed) >= 0) {
                  this.addFailureAtNode(node, Rule$2.FAILURE_STRING.replace("DISALLOWED_TEXT", disallowed));
                  // one error per line is enough!
                  return;
              }
          }
      };
      return ImportsWalker;
  }(Lint.RuleWalker));

  var tsfFoldersImportFromDisallowedFoldersRule = /*#__PURE__*/Object.freeze({
    Rule: Rule$2
  });

  var DISALLOW_IMPORT_FROM_SELF_MESSAGE = "do not import a package from itself - use a relative path";
  var DISALLOW_IMPORT_FROM_BANNED_MESSAGE = "do not use a banned import path from package";
  var Rule$3 = /** @class */ (function (_super) {
      __extends(Rule, _super);
      function Rule() {
          return _super !== null && _super.apply(this, arguments) || this;
      }
      Rule.prototype.apply = function (sourceFile) {
          var walker = new ImportsWalker$1(sourceFile, this.getOptions());
          this.applyWithWalker(walker);
          return walker.getFailures();
      };
      return Rule;
  }(Lint.Rules.AbstractRule));
  var ImportsWalker$1 = /** @class */ (function (_super) {
      __extends(ImportsWalker, _super);
      function ImportsWalker() {
          return _super !== null && _super.apply(this, arguments) || this;
      }
      ImportsWalker.prototype.visitImportDeclaration = function (node) {
          this.validate(node, node.moduleSpecifier.getText());
      };
      ImportsWalker.prototype.visitImportEqualsDeclaration = function (node) {
          this.validate(node, node.moduleReference.getText());
          _super.prototype.visitImportEqualsDeclaration.call(this, node);
      };
      ImportsWalker.prototype.validate = function (node, text) {
          // algorithm:
          /*
          - determine this files PackageFolder, PackageSubFolder
          - if ThirdParty then skip
          - determine the PackageFolder, PackageSubFolder of the import
          - if ThirdParty then skip
          - disallowImportFromSelf -> disallow if same
          - check if the import is allowed
          */
          var config = exports.ConfigFactory.createForBetweenPackages(this.getOptions());
          var thisPackageLocation = ImportRuleUtils.determinePackageLocationFromPath(node.getSourceFile().fileName, RuleId.TsfFoldersImportsBetweenPackages, config, PathSource.SourceFilePath);
          if (ImportRuleUtils.isThisPackageThirdParty(thisPackageLocation, node)) {
              return;
          }
          if (!thisPackageLocation.packageFolder) {
              throw new Error("unexpected: this package is not ThirdParty, but has no packageFolder in its location");
          }
          var importPackageLocation = ImportRuleUtils.determinePackageLocationFromPath(text, RuleId.TsfFoldersImportsBetweenPackages, config, PathSource.ImportText, thisPackageLocation);
          ImportRuleUtils.logPackageAndImport(node, thisPackageLocation, importPackageLocation);
          var isImportRecognised = !ImportRuleUtils.isPackageThirdParty(importPackageLocation);
          if (isImportRecognised &&
              importPackageLocation.packageName === thisPackageLocation.packageName &&
              config.disallowImportFromSelf.enabled &&
              !ImportRuleUtils.shouldIgnoreFile(node, config.disallowImportFromSelf.ignorePaths)) {
              this.addFailureAtNode(node, GeneralRuleUtils.buildFailureString(DISALLOW_IMPORT_FROM_SELF_MESSAGE, RuleId.TsfFoldersImportsBetweenPackages));
              return;
          }
          if (isImportRecognised &&
              config.checkImportsBetweenPackages.enabled &&
              !ImportRuleUtils.shouldIgnoreFile(node, config.checkImportsBetweenPackages.ignorePaths)) {
              if (!thisPackageLocation.packageFolder || !importPackageLocation.packageFolder) {
                  return;
              }
              if (this.hasBannedImportPattern(config.checkImportsBetweenPackages, text, node)) {
                  return;
              }
              if (importPackageLocation.packageFolder === thisPackageLocation.packageFolder) {
                  if (!config.checkImportsBetweenPackages.checkSubFoldersEnabled) {
                      return;
                  }
                  // TODO xxx extract fun?
                  // check sub-folders
                  if (thisPackageLocation.packageSubFolder &&
                      importPackageLocation.packageSubFolder &&
                      thisPackageLocation.packageSubFolder.importPath !==
                          importPackageLocation.packageSubFolder.importPath) {
                      if (thisPackageLocation.packageSubFolder.allowedToImport.some(function (allowed) { return allowed === "*"; })) {
                          return;
                      }
                      if (!thisPackageLocation.packageSubFolder.allowedToImport.some(function (allowed) {
                          return importPackageLocation.packageSubFolder.importPath === allowed;
                      })) {
                          var failureMessage = "'" + thisPackageLocation.packageName + "' sub folder '" + thisPackageLocation.packageSubFolder.importPath + "' is not allowed to import from '" + importPackageLocation.packageSubFolder.importPath + "'";
                          this.addFailureAtNodeWithMessage(node, failureMessage);
                      }
                  }
              }
              else {
                  var thisPackageFolder = thisPackageLocation.packageFolder;
                  if (thisPackageFolder.allowedToImport.find(function (allowed) { return allowed === "*"; })) {
                      return;
                  }
                  if (!thisPackageFolder.allowedToImport.find(function (allowed) { return allowed === importPackageLocation.packageName; })) {
                      var failureMessage = "'" + thisPackageLocation.packageName + "' is not allowed to import from '" + importPackageLocation.packageName + "'";
                      this.addFailureAtNodeWithMessage(node, failureMessage);
                  }
              }
          }
      };
      ImportsWalker.prototype.hasBannedImportPattern = function (checkImportsBetweenPackages, text, node) {
          var bannedImports = this.buildListOfBannedImports(checkImportsBetweenPackages);
          if (bannedImports && bannedImports.some(function (ban) { return text.indexOf(ban) >= 0; })) {
              this.addFailureAtNode(node, GeneralRuleUtils.buildFailureString(DISALLOW_IMPORT_FROM_BANNED_MESSAGE, RuleId.TsfFoldersImportsBetweenPackages));
              return true;
          }
          return false;
      };
      ImportsWalker.prototype.buildListOfBannedImports = function (checkImportsBetweenPackages) {
          var ban = checkImportsBetweenPackages.ban, banBlacklist = checkImportsBetweenPackages.banBlacklist, packages = checkImportsBetweenPackages.packages;
          if (!ban) {
              return [];
          }
          var bannedImports = [];
          packages
              .filter(function (pkg) { return !banBlacklist || !banBlacklist.some(function (b) { return pkg.importPath === b; }); })
              .forEach(function (pkg) {
              ban.forEach(function (b) {
                  bannedImports.push(b.replace("{PACKAGE}", pkg.importPath));
              });
          });
          return bannedImports;
      };
      ImportsWalker.prototype.addFailureAtNodeWithMessage = function (node, failureMessage) {
          this.addFailureAtNode(node, GeneralRuleUtils.buildFailureString(failureMessage, RuleId.TsfFoldersImportsBetweenPackages));
      };
      return ImportsWalker;
  }(Lint.RuleWalker));

  var tsfFoldersImportsBetweenPackagesRule = /*#__PURE__*/Object.freeze({
    Rule: Rule$3
  });

  var Rule$4 = /** @class */ (function (_super) {
      __extends(Rule, _super);
      function Rule() {
          return _super !== null && _super.apply(this, arguments) || this;
      }
      Rule.prototype.apply = function (sourceFile) {
          var config = exports.ConfigFactory.createForTestBreakpointRule(this.getOptions());
          if (!GeneralRuleUtils.isFileInPaths(sourceFile.fileName, config.includePaths)) {
              return [];
          }
          var walker = new StatementsWalker$1(sourceFile, this.getOptions(), config);
          this.applyWithWalker(walker);
          return walker.getFailures();
      };
      return Rule;
  }(Lint.Rules.AbstractRule));
  var StatementsWalker$1 = /** @class */ (function (_super) {
      __extends(StatementsWalker, _super);
      function StatementsWalker(sourceFile, options, config) {
          var _this = _super.call(this, sourceFile, options) || this;
          _this.config = config;
          return _this;
      }
      StatementsWalker.prototype.visitCallExpression = function (node) {
          var text = node.getText();
          if (this.config.debugTokens.some(function (token) { return text.startsWith(token); })) {
              this.addFailureAtNode(node.getFirstToken(), GeneralRuleUtils.buildFailureString("do not hard code breakpoints in the test", RuleId.TsfFoldersTestWithBreakpoint));
          }
          _super.prototype.visitCallExpression.call(this, node);
      };
      return StatementsWalker;
  }(Lint.RuleWalker));

  var tsfFoldersTestWithBreakpointRule = /*#__PURE__*/Object.freeze({
    Rule: Rule$4
  });

  exports.TtsfFoldersDisabledTestRule = tsfFoldersDisabledTestRule;
  exports.TsfFoldersFileNamesRule = tsfFoldersFileNamesRule;
  exports.TsfFoldersImportFromDisallowedFoldersRule = tsfFoldersImportFromDisallowedFoldersRule;
  exports.TsfFoldersImportsBetweenPackagesRule = tsfFoldersImportsBetweenPackagesRule;
  exports.TsfFoldersTestWithBreakpointRule = tsfFoldersTestWithBreakpointRule;

  Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=tslint-folders.umd.js.map
